@@ -207,37 +207,9 @@ class ArgoStep(ArgoObjectDict):
         self.outputs.artifacts[name].modified = {"old_key": old_key}
 
     def retry(self):
-        from .workflow import Workflow, get_argo_api_client
+        from .workflow import Workflow
         wf = Workflow(id=self.workflow)
-        assert wf.query_status() == "Running"
-        logger.info("Suspend workflow %s..." % self.workflow)
-        wf.suspend()
-        time.sleep(5)
-
-        logger.info("Query workflow %s..." % self.workflow)
-        wf_info = wf.query().recover()
-        nodes = wf_info["status"]["nodes"]
-        patch = {"status": {"nodes": {}}}
-        patch["status"]["nodes"][self.id] = {"phase": "Pending"}
-        for node in nodes.values():
-            if node["name"] != self.name and self.name.startswith(
-                    node["name"]) and node["phase"] == "Failed":
-                patch["status"]["nodes"][node["id"]] = {"phase": "Running"}
-
-        logger.info("Delete pod of step %s..." % self.id)
-        self.delete_pod()
-        with get_argo_api_client() as api_client:
-            logger.info("Update workflow %s..." % self.workflow)
-            api_client.call_api(
-                '/api/v1/workflows/%s/%s' % (
-                    config["namespace"], self.workflow),
-                'PUT', response_type='object',
-                header_params=config["http_headers"],
-                body={"patch": json.dumps(patch)},
-                _return_http_data_only=True)
-
-        logger.info("Resume workflow %s..." % self.workflow)
-        wf.resume()
+        wf.retry_steps([self.id])
 
     def get_pod(self):
         assert self.type == "Pod"
