@@ -75,6 +75,9 @@ config = {
     "debug_s3": boolize(os.environ.get("DFLOW_DEBUG_S3", False)),
     "debug_workdir": os.environ.get("DFLOW_DEBUG_WORKDIR", "."),
     "debug_artifact_dir": os.environ.get("DFLOW_DEBUG_ARTIFACT_DIR", "."),
+    "debug_failfast": boolize(os.environ.get("DFLOW_DEBUG_FAILFAST", False)),
+    "debug_save_copy_method": os.environ.get("DFLOW_DEBUG_SAVE_COPY_METHOD",
+                                             "symlink"),
 }
 
 
