@@ -1,5 +1,4 @@
 import json
-import os
 import tempfile
 from collections import UserDict
 from copy import copy, deepcopy
@@ -286,7 +285,7 @@ class ArgoVar:
 
     def __add__(self, other):
         if isinstance(other, str):
-            return ArgoVar("%s + '%s'" % (self.expr, other))
+            return str(self) + other
         if config["mode"] == "debug":
             return ArgoVar("%s + %s" % (self.expr, to_expr(other)))
         if isinstance(other, ArgoVar):
@@ -446,11 +445,11 @@ class InputParameter(ArgoVar):
 
     @classmethod
     def from_dict(cls, d):
+        _type = json.loads(d.get("description", "{}")).get("type")
         kwargs = {
             "name": d.get("name", None),
             # for backward compatible
-            "type": str if json.loads(d.get("description", "{}")).get(
-                "type", "str") in ["str", str(str)] else None,
+            "type": str if _type in ["str", str(str)] else _type,
         }
         if "value" in d:
             kwargs["value"] = d["value"]
@@ -854,12 +853,12 @@ class OutputParameter(ArgoVar):
 
     @classmethod
     def from_dict(cls, d):
+        _type = json.loads(d.get("description", "{}")).get("type")
         kwargs = {
             "name": d.get("name", None),
             "value_from_path": d.get("valueFrom", {}).get("path", None),
             # for backward compatible
-            "type": str if json.loads(d.get("description", "{}")).get(
-                "type", "str") in ["str", str(str)] else None,
+            "type": str if _type in ["str", str(str)] else _type,
             "global_name": d.get("globalName", None),
             "value_from_expression": d.get("valueFrom", {}).get("expression",
                                                                 None),
@@ -1091,7 +1090,7 @@ class OutputArtifact(ArgoVar):
 
     def __init__(
             self,
-            path: Optional[os.PathLike] = None,
+            path: Optional[str] = None,
             _from: Union[InputArtifact, "OutputArtifact"] = None,
             name: Optional[str] = None,
             step=None,
@@ -1287,6 +1286,8 @@ class OutputArtifact(ArgoVar):
         elif self.from_expression is not None:
             return V1alpha1Artifact(
                 from_expression=str(self.from_expression), **kwargs)
+        elif self.optional:
+            return V1alpha1Artifact(**kwargs)
         else:
             raise RuntimeError("Output artifact %s is not specified" % self)
 
