@@ -5,7 +5,6 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, Type, Union
 
 import jsonpickle
-import typeguard
 
 from .. import __path__
 from ..common import S3Artifact
@@ -46,6 +45,8 @@ class Slices:
             to handle each slice, 1 for serial, -1 for infinity (i.e. equals to
             the number of slices)
         register_first_only: only register first slice when lineage used
+        create_dir: create a separate dir for each slice for saving output
+            artifacts
     """
 
     def __init__(
@@ -61,6 +62,7 @@ class Slices:
             random_seed: int = 0,
             pool_size: Optional[int] = None,
             register_first_only: bool = False,
+            create_dir: bool = False,
     ) -> None:
         self.input_parameter = input_parameter if input_parameter is not \
             None else []
@@ -82,6 +84,7 @@ class Slices:
         self.random_seed = random_seed
         self.pool_size = pool_size
         self.register_first_only = register_first_only
+        self.create_dir = create_dir
 
     def evalable_repr(self, imports):
         kwargs = {}
@@ -210,6 +213,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                  pre_script: str = "",
                  post_script: str = "",
                  success_tag: bool = False,
+                 create_slice_dir: bool = False,
                  ) -> None:
         self.n_parts = {}
         self.keys_of_parts = {}
@@ -338,7 +342,6 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         if upload_dflow:
             python_packages += __path__
             python_packages += jsonpickle.__path__
-            python_packages += typeguard.__path__
 
         self.python_packages = None
         if python_packages:
@@ -373,6 +376,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             else output_artifact_slices
         self.output_parameter_slices = {} if output_parameter_slices is None \
             else output_parameter_slices
+        self.create_slice_dir = create_slice_dir
         self.set_slices(slices)
         self.download_method = "download"
 
@@ -405,8 +409,15 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                 self.input_parameter_slices[name] = slices.slices
         if slices.output_artifact:
             for name in slices.output_artifact:
-                self.output_artifact_slices[name] = slices.slices
+                if name in self.output_artifact_slices:
+                    s = str(self.output_artifact_slices[name]).strip("'")
+                    s += "." + slices.slices
+                    self.output_artifact_slices[name] = "'%s'" % s
+                else:
+                    self.output_artifact_slices[name] = slices.slices
                 self.outputs.artifacts[name].archive = None  # no archive
+                if slices.create_dir:
+                    self.create_slice_dir = True
         if slices.output_parameter:
             for name in slices.output_parameter:
                 self.output_parameter_slices[name] = slices.slices
@@ -489,17 +500,15 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         script += "op_obj.key = '{{=inputs.parameters.dflow_key}}'\n"
         script += "if op_obj.key.startswith('{'): op_obj.key = None\n"
         script += "op_obj.workflow_name = '{{workflow.name}}'\n"
-        script += "def try_to_execute(input):\n"
-        script += "    try:\n"
-        script += "        return op_obj.execute(input), None\n"
-        script += "    except Exception as e:\n"
-        script += "        traceback.print_exc()\n"
-        script += "        return None, e\n\n"
         script += "if __name__ == '__main__':\n"
         script += "    input = OPIO()\n"
         script += "    input_sign = %s.get_input_sign()\n" % class_name
         script += "    output_sign = %s.get_output_sign()\n" % class_name
         if self.slices is not None and self.slices.pool_size is not None:
+            script += "    from dflow.python.utils import try_to_execute\n"
+            script += "    from functools import partial\n"
+            script += "    try_to_execute = partial(try_to_execute, "\
+                "op_obj=op_obj, output_sign=output_sign, cwd=os.getcwd())\n"
             script += "    from typing import List\n"
             script += "    from pathlib import Path\n"
             for name in self.slices.input_artifact:
@@ -521,6 +530,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                         "inputs/artifacts/%s'))\n" % (
                             name, self.download_method, name, self.tmp_root,
                             name)
+        script += "    is_outputs = False\n"
         for name, sign in input_sign.items():
             if isinstance(sign, Artifact):
                 slices = self.get_slices(input_artifact_slices, name)
@@ -571,11 +581,20 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                 script += "        input1['%s'] = list(input['%s'])[i] if "\
                     "input['%s'] is not None else None\n" % (name, name, name)
             script += "        input_list.append(input1)\n"
+            if self.create_slice_dir:
+                slices = self.render_slices(self.slices.slices)
+                script += "    from dflow.python.utils import slice_to_dir\n"
+                script += "    slice_dirs = list(map(slice_to_dir, %s))\n" % \
+                    slices
+            else:
+                script += "    slice_dirs = [None] * n_slices\n"
             if self.slices.pool_size == 1:
                 script += "    output_list = []\n"
                 script += "    error_list = []\n"
-                script += "    for input in input_list:\n"
-                script += "        output, error = try_to_execute(input)\n"
+                script += "    for input, slice_dir in zip(input_list, "\
+                    "slice_dirs):\n"
+                script += "        output, error = try_to_execute(input, "\
+                    "slice_dir)\n"
                 script += "        output_list.append(output)\n"
                 script += "        error_list.append(error)\n"
             else:
@@ -585,8 +604,8 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                 else:
                     script += "    pool = Pool(%s)\n" % \
                         self.slices.pool_size
-                script += "    oe_list = pool.map(try_to_execute, "\
-                    "input_list)\n"
+                script += "    oe_list = pool.starmap(try_to_execute, "\
+                    "zip(input_list, slice_dirs))\n"
                 script += "    output_list = [oe[0] for oe in oe_list]\n"
                 script += "    error_list = [oe[1] for oe in oe_list]\n"
             sliced_outputs = self.slices.output_artifact + \
@@ -619,12 +638,15 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             % self.tmp_root
         script += "    os.makedirs(r'%s/outputs/artifacts', exist_ok=True)\n" \
             % self.tmp_root
+        script += "    is_outputs = True\n"
         for name, sign in output_sign.items():
             if isinstance(sign, Artifact):
                 slices = self.get_slices(output_artifact_slices, name)
                 script += "    handle_output_artifact('%s', output['%s'], "\
-                    "output_sign['%s'], %s, r'%s')\n" % (name, name, name,
-                                                         slices, self.tmp_root)
+                    "output_sign['%s'], %s, r'%s', %s)\n" % (
+                        name, name, name, slices, self.tmp_root,
+                        slices is not None and self.create_slice_dir and
+                        getattr(self.slices, "pool_size", None) is None)
             else:
                 slices = self.get_slices(output_parameter_slices, name)
                 script += "    handle_output_parameter('%s', output['%s'], "\
@@ -741,8 +763,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         del g["resource"]
         python_packages = []
         for p in self.python_packages:
-            if self.upload_dflow and Path(p).name in [
-                    "dflow", "jsonpickle", "typeguard"]:
+            if self.upload_dflow and Path(p).name in ["dflow", "jsonpickle"]:
                 continue
             python_packages.append(str(p))
         g["type"] = "PythonOPTemplate"
