@@ -1,5 +1,6 @@
 import os
 import shutil
+import traceback
 import uuid
 from pathlib import Path
 from typing import Dict, List, Set
@@ -161,37 +162,41 @@ def handle_input_parameter(name, value, sign, slices=None, data_root="/tmp"):
     return obj
 
 
-def handle_output_artifact(name, value, sign, slices=None, data_root="/tmp"):
+def slice_to_dir(slice):
+    return str(slice).replace(".", "/")
+
+
+def handle_output_artifact(name, value, sign, slices=None, data_root="/tmp",
+                           create_dir=False):
     path_list = []
     if sign.type in [str, Path]:
         os.makedirs(data_root + '/outputs/artifacts/' + name, exist_ok=True)
-        if slices is not None:
-            assert isinstance(slices, int)
-        else:
+        if slices is None:
             slices = 0
-        if value and os.path.exists(str(value)):
-            path_list.append({"dflow_list_item": copy_results(
-                value, name, data_root), "order": slices})
-        else:
-            path_list.append({"dflow_list_item": None, "order": slices})
+        path_list.append(copy_results_and_return_path_item(
+            value, name, slices, data_root,
+            slice_to_dir(slices) if create_dir else None))
     elif sign.type in [List[str], List[Path], Set[str], Set[Path]]:
         os.makedirs(data_root + '/outputs/artifacts/' + name, exist_ok=True)
         if slices is not None:
             if isinstance(slices, int):
                 for path in value:
                     path_list.append(copy_results_and_return_path_item(
-                        path, name, slices, data_root))
+                        path, name, slices, data_root,
+                        slice_to_dir(slices) if create_dir else None))
             else:
                 assert len(slices) == len(value)
                 for path, s in zip(value, slices):
                     if isinstance(path, list):
                         for p in path:
                             path_list.append(
-                                copy_results_and_return_path_item(p, name, s,
-                                                                  data_root))
+                                copy_results_and_return_path_item(
+                                    p, name, s, data_root, slice_to_dir(
+                                        s) if create_dir else None))
                     else:
                         path_list.append(copy_results_and_return_path_item(
-                            path, name, s, data_root))
+                            path, name, s, data_root, slice_to_dir(
+                                s) if create_dir else None))
         else:
             for s, path in enumerate(value):
                 path_list.append(copy_results_and_return_path_item(
@@ -244,15 +249,16 @@ def handle_output_parameter(name, value, sign, slices=None, data_root="/tmp"):
                 f.write(jsonpickle.dumps(value))
 
 
-def copy_results_and_return_path_item(path, name, order, data_root="/tmp"):
+def copy_results_and_return_path_item(path, name, order, data_root="/tmp",
+                                      slice_dir=None):
     if path and os.path.exists(str(path)):
-        return {"dflow_list_item": copy_results(path, name, data_root),
-                "order": order}
+        return {"dflow_list_item": copy_results(
+                    path, name, data_root, slice_dir), "order": order}
     else:
         return {"dflow_list_item": None, "order": order}
 
 
-def copy_results(source, name, data_root="/tmp"):
+def copy_results(source, name, data_root="/tmp", slice_dir=None):
     source = str(source)
     # if refer to input artifact
     if source.find(data_root + "/inputs/artifacts/") == 0:
@@ -262,17 +268,25 @@ def copy_results(source, name, data_root="/tmp"):
             rel_path = randstr()
         else:
             rel_path = source[i+1:]
+        if slice_dir is not None:
+            rel_path = "%s/%s" % (slice_dir, rel_path)
         target = data_root + "/outputs/artifacts/%s/%s" % (name, rel_path)
         copy_file(source, target, shutil.copy)
         if rel_path[:1] == "/":
             rel_path = rel_path[1:]
         return rel_path
     else:
-        target = data_root + "/outputs/artifacts/%s/%s" % (name, source)
+        cwd = os.getcwd()
+        if not cwd.endswith("/"):
+            cwd = cwd + "/"
+        if source.startswith(cwd):
+            source = source[len(cwd):]
+        rel_path = source[1:] if source[:1] == "/" else source
+        if slice_dir is not None:
+            rel_path = "%s/%s" % (slice_dir, rel_path)
+        target = data_root + "/outputs/artifacts/%s/%s" % (name, rel_path)
         copy_file(source, target)
-        if source[:1] == "/":
-            source = source[1:]
-        return source
+        return rel_path
 
 
 def handle_empty_dir(path):
@@ -300,3 +314,34 @@ def handle_lineage(wf_name, pod_name, op_obj, input_urns, workflow_urn,
         with open("%s/outputs/parameters/dflow_%s_urn" % (data_root, name),
                   "w") as f:
             f.write(urn)
+
+
+def absolutize(path):
+    if path is None:
+        return None
+    if isinstance(path, str):
+        return os.path.abspath(path)
+    if isinstance(path, Path):
+        return path.absolute()
+    if isinstance(path, list):
+        return [absolutize(p) for p in path]
+    if isinstance(path, dict):
+        return {k: absolutize(p) for k, p in path.items()}
+
+
+def try_to_execute(input, slice_dir, op_obj, output_sign, cwd):
+    os.chdir(cwd)
+    if slice_dir is not None:
+        os.makedirs(slice_dir, exist_ok=True)
+        os.chdir(slice_dir)
+    try:
+        output = op_obj.execute(input)
+        for n, s in output_sign.items():
+            if isinstance(s, Artifact):
+                output[n] = absolutize(output[n])
+        os.chdir(cwd)
+        return output, None
+    except Exception as e:
+        traceback.print_exc()
+        os.chdir(cwd)
+        return None, e
