@@ -1,4 +1,5 @@
 import abc
+import concurrent.futures
 import contextlib
 import hashlib
 import inspect
@@ -394,7 +395,9 @@ def upload_s3(
     else:
         client = MinioClient(**kwargs)
     if key is not None:
-        pass
+        if not key.startswith(s3_config["prefix"]) and not any(
+                [key.startswith(p) for p in s3_config["extra_prefixes"]]):
+            key = s3_config["prefix"] + key
     elif prefix is not None:
         if prefix[-1] != "/":
             prefix += "/"
@@ -409,7 +412,7 @@ def upload_s3(
         client.upload(key=key, path=path)
     elif os.path.isdir(path):
         for dn, ds, fs in os.walk(path, followlinks=True):
-            rel_path = dn[len(path):]
+            rel_path = dn[len(str(path)):]
             if rel_path == "":
                 pass
             elif rel_path[0] != "/":
@@ -622,8 +625,10 @@ def set_directory(dirname: os.PathLike, mkdir: bool = False):
     if mkdir:
         path.mkdir(exist_ok=True, parents=True)
     os.chdir(path)
-    yield path
-    os.chdir(pwd)
+    try:
+        yield path
+    finally:
+        os.chdir(pwd)
 
 
 def run_command(
@@ -631,7 +636,9 @@ def run_command(
     raise_error: bool = True,
     input: Optional[str] = None,
     try_bash: bool = False,
+    login: bool = True,
     interactive: bool = True,
+    shell: bool = False,
     print_oe: bool = False,
     **kwargs,
 ) -> Tuple[int, str, str]:
@@ -648,6 +655,14 @@ def run_command(
         Input string for the command
     try_bash: bool
         Try to use bash if bash exists, otherwise use sh
+    login: bool
+        Login mode of bash when try_bash=True
+    interactive: bool
+        Alias of login
+    shell: bool
+        Use shell for subprocess.Popen
+    print_oe: bool
+        Print stdout and stderr at the same time
     **kwargs:
         Arguments in subprocess.Popen
 
@@ -666,40 +681,48 @@ def run_command(
         stderr content of the executed command
     """
     if isinstance(cmd, str):
-        cmd = cmd.split()
+        if shell:
+            cmd = [cmd]
+        else:
+            cmd = cmd.split()
     elif isinstance(cmd, list):
         cmd = [str(x) for x in cmd]
 
     if try_bash:
-        arg = "-lc" if interactive else "-c"
+        arg = "-lc" if (login and interactive) else "-c"
         script = "if command -v bash 2>&1 >/dev/null; then bash %s " % arg + \
             shlex.quote(" ".join(cmd)) + "; else " + " ".join(cmd) + "; fi"
         cmd = [script]
-        kwargs["shell"] = True
+        shell = True
 
     with subprocess.Popen(
         args=cmd,
         stdin=subprocess.PIPE,
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
+        shell=shell,
         **kwargs,
     ) as sub:
-        if input is not None:
-            sub.stdin.write(bytes(input, encoding=sys.stdout.encoding))
-            sub.stdin.close()
         if print_oe:
+            if input is not None:
+                sub.stdin.write(bytes(input, encoding=sys.stdout.encoding))
+                sub.stdin.close()
             out = ""
             err = ""
             sel = selectors.DefaultSelector()
             sel.register(sub.stdout, selectors.EVENT_READ)
             sel.register(sub.stderr, selectors.EVENT_READ)
-            ok = True
-            while ok:
+            stdout_eof = False
+            stderr_eof = False
+            while not (stdout_eof and stderr_eof):
                 for key, _ in sel.select():
                     line = key.fileobj.readline().decode(sys.stdout.encoding)
                     if not line:
-                        ok = False
-                        break
+                        if key.fileobj is sub.stdout:
+                            stdout_eof = True
+                        if key.fileobj is sub.stderr:
+                            stderr_eof = True
+                        continue
                     if key.fileobj is sub.stdout:
                         sys.stdout.write(line)
                         out += line
@@ -708,7 +731,8 @@ def run_command(
                         err += line
             sub.wait()
         else:
-            out, err = sub.communicate()
+            out, err = sub.communicate(bytes(
+                input, encoding=sys.stdout.encoding) if input else None)
             out = out.decode(sys.stdout.encoding)
             err = err.decode(sys.stdout.encoding)
         return_code = sub.poll()
@@ -885,3 +909,13 @@ def evalable_repr(obj, imports):
         return obj.evalable_repr(imports)
     imports.add((None, "jsonpickle"))
     return "jsonpickle.loads(%s)" % repr(jsonpickle.dumps(obj))
+
+
+class ProcessPoolExecutor(concurrent.futures.ProcessPoolExecutor):
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        # Work around of getting stuck when exiting, only work for python>=3.9
+        if sys.version_info.minor >= 9:
+            self.shutdown(wait=False)
+        else:
+            self.shutdown(wait=True)
+        return False
