@@ -40,6 +40,7 @@ try:
                                        V1PersistentVolumeClaimSpec,
                                        V1ResourceRequirements,
                                        WorkflowServiceApi)
+    from argo.workflows.client.exceptions import ApiException
 except Exception:
     pass
 
@@ -370,7 +371,7 @@ class Workflow:
         while self.query_status() in ["Pending", "Running"]:
             time.sleep(interval)
 
-    def handle_reused_step(self, step):
+    def handle_reused_step(self, step, global_parameters, global_artifacts):
         outputs = {}
         if hasattr(step, "outputs"):
             if hasattr(step.outputs, "exitCode"):
@@ -378,25 +379,36 @@ class Workflow:
             if hasattr(step.outputs, "parameters"):
                 outputs["parameters"] = []
                 for name, par in step.outputs.parameters.items():
-                    if not hasattr(step.outputs.parameters[name],
-                                   "save_as_artifact"):
+                    if not hasattr(par, "save_as_artifact"):
                         outputs["parameters"].append(par.recover())
+                        if hasattr(par, "globalName") and name != \
+                                "dflow_global":
+                            global_par = par.recover()
+                            global_par["name"] = par.globalName
+                            global_par.pop("globalName", None)
+                            global_parameters[par.globalName] = global_par
             if hasattr(step.outputs, "artifacts"):
                 for name, art in step.outputs.artifacts.items():
                     group_key = step.get("inputs", {}).get(
                         "parameters", {}).get("dflow_group_key", {}).get(
                         "value")
                     art_key = get_key(art, raise_error=False)
-                    if group_key and art_key and art_key.endswith(
-                            "%s/%s" % (group_key, name)):
+                    if group_key and ((art_key and art_key.endswith(
+                        "%s/%s" % (group_key, name))) or (
+                        hasattr(art, "modified") and art.modified[
+                            "old_key"].endswith("%s/%s" % (group_key, name)))):
                         if config["overwrite_reused_artifact"]:
                             self.handle_reused_artifact(step, name, art)
                         else:
                             self.handle_reused_artifact_with_copy(
                                 step, name, art)
-
-        outputs["artifacts"] = [
-            art.recover() for art in step.outputs.artifacts.values()]
+                    if hasattr(art, "globalName"):
+                        global_art = art.recover()
+                        global_art["name"] = art.globalName
+                        global_art.pop("globalName", None)
+                        global_artifacts[art.globalName] = global_art
+                outputs["artifacts"] = [
+                    art.recover() for art in step.outputs.artifacts.values()]
         self.memoize_map["%s-%s" % (self.id, step.key)] = {
             "nodeID": step.id,
             "outputs": outputs,
@@ -468,7 +480,8 @@ class Workflow:
             assert isinstance(self.context, (Context, Executor))
             self = self.context.render(self)
 
-        status = None
+        global_parameters = {}
+        global_artifacts = {}
         if reuse_step is not None:
             self.reused_keys = [step.key for step in reuse_step
                                 if step.key is not None]
@@ -482,13 +495,17 @@ class Workflow:
                 if step.key is None:
                     continue
                 key2id[step.key] = step.id
-                self.handle_reused_step(step)
+                self.handle_reused_step(step, global_parameters,
+                                        global_artifacts)
 
             for key, step in self.memoize_map.items():
                 data = {key: json.dumps(step)}
                 config_map = kubernetes.client.V1ConfigMap(
                     data=data, metadata=kubernetes.client.V1ObjectMeta(
-                        name="dflow-%s" % key))
+                        name="dflow-%s" % key,
+                        labels={
+                            "workflows.argoproj.io/configmap-type": "Cache",
+                        }))
                 core_v1_api = self.get_k8s_core_v1_api()
                 logger.debug("creating configmap: %s" %
                              config_map.metadata.name)
@@ -501,8 +518,8 @@ class Workflow:
             self.handle_template(self.entrypoint, memoize_prefix=self.id,
                                  memoize_configmap="dflow")
             if config["save_keys_in_global_outputs"]:
-                status = {"outputs": {"parameters": [
-                    {"name": key, "value": id} for key, id in key2id.items()]}}
+                for key, id in key2id.items():
+                    global_parameters[key] = {"name": key, "value": id}
         else:
             self.handle_template(self.entrypoint)
 
@@ -568,7 +585,8 @@ class Workflow:
                 artifact_repository_ref=None if self.artifact_repo_key is None
                 else V1alpha1ArtifactRepositoryRef(key=self.artifact_repo_key)
             ),
-            status=status)
+            status={"outputs": {"parameters": list(global_parameters.values()),
+                                "artifacts": list(global_artifacts.values())}})
 
     def deduplicate_templates(self):
         logger.debug("before deduplication: %s" % len(self.argo_templates))
@@ -830,6 +848,7 @@ class Workflow:
     def query(
             self,
             fields: Optional[List[str]] = None,
+            retry: int = 3,
     ) -> ArgoWorkflow:
         """
         Query the workflow from Argo
@@ -841,6 +860,31 @@ class Workflow:
         Returns:
             an ArgoWorkflow object
         """
+        if config["mode"] == "debug":
+            nodes = {}
+            for step in self.query_step():
+                step.inputs.parameters = list(step.inputs.parameters.values())
+                step.inputs.artifacts = list(step.inputs.artifacts.values())
+                step.outputs.parameters = list(
+                    step.outputs.parameters.values())
+                step.outputs.artifacts = list(step.outputs.artifacts.values())
+                nodes[step.id] = step.recover()
+            outputs = self.query_global_outputs()
+            if outputs is not None:
+                outputs.parameters = list(outputs.parameters.values())
+                outputs.artifacts = list(outputs.artifacts.values())
+                outputs = outputs.recover()
+            response = {
+                "metadata": {
+                    "name": self.id,
+                },
+                "status": {
+                    "phase": self.query_status(),
+                    "nodes": nodes,
+                    "outputs": outputs,
+                }
+            }
+            return ArgoWorkflow(response)
         query_params = None
         if fields is not None:
             query_params = [('fields', ",".join(fields))]
@@ -850,12 +894,20 @@ class Workflow:
                 'GET', response_type=object, _return_http_data_only=True,
                 header_params=config["http_headers"],
                 query_params=query_params)
-        except Exception:
-            response = self.api_instance.api_client.call_api(
-                '/api/v1/archived-workflows/%s' % self.uid,
-                'GET', response_type=object, _return_http_data_only=True,
-                header_params=config["http_headers"],
-                query_params=query_params)
+        except ApiException as e:
+            if e.status == 404:
+                response = self.api_instance.api_client.call_api(
+                    '/api/v1/archived-workflows/%s' % self.uid,
+                    'GET', response_type=object, _return_http_data_only=True,
+                    header_params=config["http_headers"],
+                    query_params=query_params)
+            elif e.status >= 500 and e.status < 600 and retry > 0:
+                logger.error("API Exception: %s" % e)
+                logger.error("Remaining retry: %s" % retry)
+                time.sleep(1)
+                return self.query(fields=fields, retry=retry-1)
+            else:
+                raise e
         workflow = ArgoWorkflow(response)
         return workflow
 
@@ -941,6 +993,7 @@ class Workflow:
                     "workflow": self.id,
                     "displayName": _name,
                     "key": s,
+                    "id": s,
                     "startedAt": os.path.getmtime(stepdir),
                     "phase": _phase,
                     "type": _type,
@@ -984,6 +1037,7 @@ class Workflow:
                             })
                 step = ArgoStep(step, self.id)
                 step_list.append(step)
+            step_list.sort(key=lambda x: x["startedAt"])
             return step_list
 
         return self.query().get_step(name=name, key=key, phase=phase, id=id,
@@ -1001,7 +1055,8 @@ class Workflow:
             a list of keys
         """
         if config["mode"] == "debug":
-            return [step.key for step in self.query_step()]
+            return [step.key for step in self.query_step()
+                    if step.key is not None]
         try:
             try:
                 response = self.api_instance.api_client.call_api(
@@ -1189,6 +1244,40 @@ class Workflow:
             '/api/v1/workflows/%s/%s/suspend' % (self.namespace, self.id),
             'PUT', header_params=config["http_headers"])
 
+    def retry_steps(self, step_ids):
+        assert self.query_status() == "Running"
+        logger.info("Suspend workflow %s..." % self.id)
+        self.suspend()
+        time.sleep(5)
+
+        logger.info("Query workflow %s..." % self.id)
+        wf_info = self.query().recover()
+        nodes = wf_info["status"]["nodes"]
+        patch = {"status": {"nodes": {}}}
+        for step_id in step_ids:
+            step = ArgoStep(nodes[step_id], self.id)
+            patch["status"]["nodes"][step_id] = {"phase": "Pending"}
+            for node in nodes.values():
+                if node["name"] != step.name and step.name.startswith(
+                        node["name"]) and node["phase"] == "Failed":
+                    patch["status"]["nodes"][node["id"]] = {"phase": "Running"}
+
+            logger.info("Delete pod of step %s..." % step_id)
+            step.delete_pod()
+
+        with get_argo_api_client() as api_client:
+            logger.info("Update workflow %s..." % self.id)
+            api_client.call_api(
+                '/api/v1/workflows/%s/%s' % (
+                    config["namespace"], self.id),
+                'PUT', response_type='object',
+                header_params=config["http_headers"],
+                body={"patch": json.dumps(patch)},
+                _return_http_data_only=True)
+
+        logger.info("Resume workflow %s..." % self.id)
+        self.resume()
+
 
 def get_argo_api_client(host=None, token=None):
     if host is None:
@@ -1234,9 +1323,12 @@ def query_workflows(labels: Optional[Dict[str, str]] = None,
 
 
 def query_archived_workflows(
-        labels: Optional[Dict[str, str]] = None) -> List[ArgoWorkflow]:
-    query_params = [('listOptions.fieldSelector',
-                     "metadata.namespace=%s" % config["namespace"])]
+        labels: Optional[Dict[str, str]] = None,
+        id: Optional[str] = None) -> List[ArgoWorkflow]:
+    sel = "metadata.namespace=%s" % config["namespace"]
+    if id is not None:
+        sel += ",metadata.name=%s" % id
+    query_params = [('listOptions.fieldSelector', sel)]
     if labels is not None:
         query_params.append((
             'listOptions.labelSelector',
